/*
 * *************************************************
 * Copyright (c) 2019 QMTH. All Rights Reserved.
 * Created by Deason on 2019-05-29 11:54:21.
 * *************************************************
 */

package cn.com.qmth.examcloud.app.core;

import cn.com.qmth.examcloud.commons.exception.ExamCloudRuntimeException;
import com.google.common.cache.Cache;
import com.google.common.cache.CacheBuilder;
import com.google.common.collect.Lists;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cloud.client.ServiceInstance;
import org.springframework.cloud.client.discovery.DiscoveryClient;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Random;
import java.util.concurrent.TimeUnit;

/**
 * 系统实例
 *
 * @author: fengdesheng
 * @since: 2019/5/29
 */
@Component
public class CloudDiscoveryClient {

    @Autowired(required = false)
    private DiscoveryClient discoveryClient;

    private Cache<String, List<String>> cache = CacheBuilder.newBuilder()
            .maximumSize(1000)//最大缓存对象数
            .expireAfterWrite(60, TimeUnit.SECONDS)//过期时间
            .concurrencyLevel(8)//并发级别,即允许最多N个线程并发更新, 默认值为4
            .recordStats()//开启记录状态
            .build();

    /**
     * 获取实例的URL
     */
    public String getInstanceUrl(String appName) {
        List<String> appUrls = cache.getIfPresent(appName);
        if (CollectionUtils.isNotEmpty(appUrls)) {
            //随机分配一个地址
            int n = this.randomInt(0, appUrls.size() - 1);
            return appUrls.get(n);
        }

        List<ServiceInstance> instances = discoveryClient.getInstances(appName);
        if (CollectionUtils.isEmpty(instances)) {
            throw new ExamCloudRuntimeException("No Instance " + appName);
        }

        appUrls = Lists.newArrayList();
        for (ServiceInstance instance : instances) {
            appUrls.add(String.format("http://%s:%s", instance.getHost(), instance.getPort()));
        }
        cache.put(appName, appUrls);

        //随机分配一个地址
        int n = this.randomInt(0, appUrls.size() - 1);
        return appUrls.get(n);
    }

    public int randomInt(int min, int max) {
        return new Random().nextInt(max - min + 1) + min;
    }

}