/*
 * *************************************************
 * Copyright (c) 2018 QMTH. All Rights Reserved.
 * Created by Deason on 2018-07-20 14:15:02.
 * *************************************************
 */

package cn.com.qmth.examcloud.app.core.config;

import cn.com.qmth.examcloud.app.model.DeviceRecord;
import cn.com.qmth.examcloud.app.service.DeviceRecordService;
import cn.com.qmth.examcloud.commons.util.ThreadLocalUtil;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.ThreadContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.servlet.handler.HandlerInterceptorAdapter;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.Date;
import java.util.Map;

import static cn.com.qmth.examcloud.app.model.Constants.*;

/**
 * API请求拦截器
 *
 * @author: fengdesheng
 * @since: 2018/7/16
 */
public class AccessInterceptor extends HandlerInterceptorAdapter {
    private final static Logger log = LoggerFactory.getLogger(AccessInterceptor.class);
    @Autowired
    private DeviceRecordService deviceRecordService;

    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) throws Exception {
        String mapping = request.getServletPath().toLowerCase();
        if (mapping.endsWith(".js") || mapping.endsWith(".css")
                || mapping.endsWith(".jpg") || mapping.endsWith(".png")
                || mapping.endsWith(".gif") || mapping.endsWith(".map")
                || mapping.endsWith(".woff") || mapping.endsWith(".ico")) {
            return true;
        }

        // 设置log4j线程上下文
        String traceId = ThreadLocalUtil.next();
        ThreadContext.put("TRACE_ID", traceId);

        if (request.getServletPath().contains("/router")) {
            return true;
        }

        //异步保存设备访问记录
        DeviceRecord record = this.parseParams(request);
        deviceRecordService.addDeviceRecord(record);
        return true;
    }

    private DeviceRecord parseParams(HttpServletRequest request) {
        String params = this.getParams(request);
        String url = request.getServletPath() + params;
        String ip = this.getIp(request);
        //封装设备访问记录信息
        DeviceRecord record = new DeviceRecord();
        record.setSystem(request.getHeader("system"));
        record.setDeviceId(request.getHeader("deviceId"));
        record.setNetType(request.getHeader("netType"));
        record.setBrand(request.getHeader("brand"));
        record.setModel(request.getHeader("model"));
        record.setSysVersion(request.getHeader("sysVersion"));
        record.setAppVersion(request.getHeader("appVersion"));
        record.setAppCode(request.getHeader("appCode"));
        record.setPatchCode(request.getHeader("patchCode"));
        record.setAccount(request.getHeader("account"));
        record.setLoginKey(request.getHeader(PARAM_APP_KEY));
        record.setLoginToken(request.getHeader(PARAM_APP_TOKEN));
        record.setUrl(url);
        record.setIp(ip);
        record.setCreateDate(new Date());
        return record;
    }

    private String getParams(HttpServletRequest request) {
        StringBuilder params = new StringBuilder();
        Map<String, String[]> map = request.getParameterMap();
        for (Map.Entry<String, String[]> entry : map.entrySet()) {
            String name = entry.getKey();
            String[] values = entry.getValue();
            params.append("&").append(name).append("=");
            if (values.length > 0) {
                if (name.toLowerCase().indexOf(PARAM_PSW) >= 0) {
                    params.append("***");
                } else {
                    params.append(values[0]);
                }
            }
        }
        return params.toString().replaceFirst("&", "?");
    }

    private String getIp(HttpServletRequest request) {
        String ip = request.getHeader("X-Forwarded-For");
        if (StringUtils.isEmpty(ip) || "unknown".equalsIgnoreCase(ip)) {
            ip = request.getHeader("Proxy-Client-IP");
        }
        if (StringUtils.isEmpty(ip) || "unknown".equalsIgnoreCase(ip)) {
            ip = request.getHeader("WL-Proxy-Client-IP");
        }
        if (StringUtils.isEmpty(ip) || "unknown".equalsIgnoreCase(ip)) {
            ip = request.getHeader("HTTP_CLIENT_IP");
        }
        if (StringUtils.isEmpty(ip) || "unknown".equalsIgnoreCase(ip)) {
            ip = request.getHeader("HTTP_X_FORWARDED_FOR");
        }
        if (StringUtils.isEmpty(ip) || "unknown".equalsIgnoreCase(ip)) {
            ip = request.getRemoteAddr();
        }
        if (StringUtils.isNotEmpty(ip)) {
            ip = ip.replaceAll("0:0:0:0:0:0:0:1", "127.0.0.1");
        }
        return ip;
    }

}