/*
 * *************************************************
 * Copyright (c) 2018 QMTH. All Rights Reserved.
 * Created by Deason on 2018-07-31 17:31:34.
 * *************************************************
 */

package cn.com.qmth.examcloud.app.service;

import cn.com.qmth.examcloud.app.model.Result;

/**
 * 网考业务服务接口
 *
 * @author: fengdesheng
 * @since: 2018/7/31
 */
public interface CoreOeService {

    /**
     * 获取服务器当前时间
     *
     * @param key
     * @param token
     * @return
     * @throws Exception
     */
    Result getCurrentTime(String key, String token) throws Exception;

    /**
     * 获取当前用户参加的离线课程列表
     *
     * @param key
     * @param token
     * @return
     * @throws Exception
     */
    Result getOfflineExamCourseList(String key, String token) throws Exception;

    /**
     * 离线考试的抽取考题
     *
     * @param key
     * @param token
     * @param examStudentId
     * @return
     * @throws Exception
     */
    Result startOfflineExamRecord(String key, String token, String examStudentId) throws Exception;

    /**
     * 上传作答文件
     *
     * @param key
     * @param token
     * @param examRecordId
     * @param fileBytes
     * @param fileName
     * @param md5
     * @return
     * @throws Exception
     */
    Result uploadPaperAnswer(String key, String token, String examRecordId, byte[] fileBytes, String fileName, String md5) throws Exception;

    /**
     * 获取某考试批次下的课程列表
     *
     * @param key
     * @param token
     * @param examId
     * @return
     * @throws Exception
     */
    Result getPracticeExamCourseList(String key, String token, String examId) throws Exception;

    /**
     * 获取当前练习的剩余作答时间
     *
     * @param key
     * @param token
     * @return
     * @throws Exception
     */
    Result getExamRecordHeartbeat(String key, String token) throws Exception;

    /**
     * 考生“开始练习”
     *
     * @param key
     * @param token
     * @param examStudentId
     * @return
     * @throws Exception
     */
    Result startPracticeExamRecord(String key, String token, String examStudentId) throws Exception;

    /**
     * 获取当前练习的试卷大题结构信息
     *
     * @param key
     * @param token
     * @param examRecordId
     * @return
     * @throws Exception
     */
    Result getAdminExamPaperStructList(String key, String token, String examRecordId,String fromCache) throws Exception;
    /**
     * 获取当前练习的试卷大题结构信息
     *
     * @param key
     * @param token
     * @param examRecordId
     * @return
     * @throws Exception
     */
    Result getStudentExamPaperStructList(String key, String token, String examRecordId) throws Exception;

    /**
     * 获取当前练习的试卷试题列表
     *
     * @param key
     * @param token
     * @return
     * @throws Exception
     */
    Result getExamRecordPaperQuestionList(String key, String token) throws Exception;

    /**
     * 获取考生作答的某个试题的详细信息
     *
     * @param key
     * @param token
     * @param questionId
     * @return
     * @throws Exception
     */
    Result getExamRecordPaperQuestionDetail(String key, String token, String questionId, String examRecordId) throws Exception;

    /**
     * 保存或更新考生作答的某个试题答案
     *
     * @param key
     * @param token
     * @param studentAnswer
     * @return
     * @throws Exception
     */
    Result updateExamRecordPaperQuestionAnswer(String key, String token, Integer order, String studentAnswer) throws Exception;

    /**
     * 当前练习的交卷
     *
     * @param key
     * @param token
     * @return
     * @throws Exception
     */
    Result submitPracticeExamRecord(String key, String token) throws Exception;

    /**
     * 检查考生当前是否有正在进行的练习记录
     *
     * @param key
     * @param token
     * @return
     * @throws Exception
     */
    Result checkOnlineExamRecord(String key, String token) throws Exception;

    /**
     * 获取当前考生的当前课程的历史练习记录
     *
     * @param key
     * @param token
     * @param examStudentId
     * @return
     * @throws Exception
     */
    Result getExamRecordPracticeHistoryList(String key, String token, String examStudentId) throws Exception;

    /**
     * 获取成绩报告的答题情况统计
     *
     * @param key
     * @param token
     * @param examRecordId
     * @return
     * @throws Exception
     */
    Result getExamRecordTotalInfo(String key, String token, String examRecordId,String fromCache) throws Exception;

    /**
     * 获取作答的题列表
     *
     * @param key
     * @param token
     * @param examRecordId
     * @return
     * @throws Exception
     */
    Result getExamRecordQuestionDetailList(String key, String token, String examRecordId) throws Exception;

    /**
     * 获取当前试题的音频已播放次数
     *
     * @param key
     * @param token
     * @param questionId
     * @return
     * @throws Exception
     */
    Result getExamRecordQuestionAudioPlayTimes(String key, String token, String questionId) throws Exception;

    /**
     * 更新当前试题的音频已播放次数
     *
     * @param key
     * @param token
     * @param questionId
     * @param mediaName
     * @return
     * @throws Exception
     */
    Result updateExamRecordQuestionAudioPlayTimes(String key, String token, String questionId, String mediaName) throws Exception;

    /**
     * 练习记录配置信息
     *
     * @param key
     * @param token
     * @param examRecordDataId
     * @return
     * @throws Exception
     */
    Result findExamRecordDataEntity(String key, String token, Long examRecordDataId,String fromCache) throws Exception;

    /**
     * 练习记录试题列表
     *
     * @param key
     * @param token
     * @param examRecordDataId
     * @return
     * @throws Exception
     */
    Result getExamRecordQuestions(String key, String token, Long examRecordDataId,String fromCache) throws Exception;

    /**查询考生的考试批次属性集
     * @param key
     * @param token
     * @param examId
     * @param keys
     * @return
     * @throws Exception
     */
    Result getExamPropertyFromCacheByStudentSession(String key, String token, Long examId, String keys)
            throws Exception;

    /**根据Id获取试卷
     * @param key
     * @param token
     * @param paperId
     * @return
     */
    Result getPaperById(String key, String token, String paperId) throws Exception ;

}