/*
 * *************************************************
 * Copyright (c) 2018 QMTH. All Rights Reserved.
 * Created by Deason on 2018-07-31 17:19:06.
 * *************************************************
 */

package cn.com.qmth.examcloud.app.service.impl;

import cn.com.qmth.examcloud.app.core.utils.HttpUtils;
import cn.com.qmth.examcloud.app.model.Result;
import cn.com.qmth.examcloud.app.service.CoreExamWorkService;
import cn.com.qmth.examcloud.app.core.SysProperty;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * 考务业务服务接口
 *
 * @author: fengdesheng
 * @since: 2018/7/16
 */
@Service
public class CoreExamWorkServiceImpl implements CoreExamWorkService {
    private static Logger log = LoggerFactory.getLogger(CoreExamWorkServiceImpl.class);
    @Autowired
    private SysProperty sysProperty;

    @Override
    public Result getPracticeExamList(String key, String token, String studentId) throws Exception {
        //封装请求参数
        final String requestUrl = String.format("%s/api/ecs_exam_work/exam/queryByNameLike?name=&examTypes=PRACTICE&studentId=%s", sysProperty.getCoreExamWorkUrl(), studentId);
        Result<String> result = HttpUtils.doGet(requestUrl, key, token);
        if (result.isSuccess()) {
            String data = result.getData();
            //替换掉图片标签内容
            data = HttpUtils.replaceImages(data);
            //过滤掉为空的属性
            data = HttpUtils.filterNullAttributes(data);
            result.setData(data);
        }
        return result;
    }

    @Override
    public Result getExamInfo(String key, String token, Long examId) throws Exception {
        //封装请求参数
        final String requestUrl = String.format("%s/api/ecs_exam_work/exam/%s", sysProperty.getCoreExamWorkUrl(), examId);
        Result<String> result = HttpUtils.doGet(requestUrl, key, token);
        if (result.isSuccess()) {
            //过滤掉为空的属性
            result.setData(HttpUtils.filterNullAttributes(result.getData()));
        }
        return result;
    }

    @Override
    public Result getBeforeExamRemark(String key, String token, Long examId, String type) throws Exception {
        //封装请求参数
        final String requestUrl = String.format("%s/api/ecs_exam_work/exam/examOrgProperty/%s/%s", sysProperty.getCoreExamWorkUrl(), examId, type);
        Result<String> result = HttpUtils.doGet(requestUrl, key, token);
        if (result.isSuccess()) {
            //过滤掉为空的属性
            result.setData(HttpUtils.filterNullAttributes(result.getData()));
        }
        return result;
    }

    @Override
    public Result getFreezeTime(String key, String token, Long examId) throws Exception {
        //封装请求参数
        final String requestUrl = String.format("%s/api/ecs_exam_work/exam/examOrgProperty/%s/FREEZE_TIME", sysProperty.getCoreExamWorkUrl(), examId);
        Result<String> result = HttpUtils.doGet(requestUrl, key, token);
        if (result.isSuccess()) {
            //过滤掉为空的属性
            result.setData(HttpUtils.filterNullAttributes(result.getData()));
        }
        return result;
    }

    @Override
    public Result getUpLoadType(String key, String token, Long examId) throws Exception {
        //封装请求参数
        final String requestUrl = String.format("%s/api/ecs_exam_work/exam/examOrgProperty/%s/OFFLINE_UPLOAD_FILE_TYPE", sysProperty.getCoreExamWorkUrl(), examId);
        Result<String> result = HttpUtils.doGet(requestUrl, key, token);
        if (result.isSuccess()) {
            //过滤掉为空的属性
            result.setData(HttpUtils.filterNullAttributes(result.getData()));
        }
        return result;
    }

}