/*
 * *************************************************
 * Copyright (c) 2018 QMTH. All Rights Reserved.
 * Created by Deason on 2018-07-31 17:19:06.
 * *************************************************
 */

package cn.com.qmth.examcloud.app.service.impl;

import cn.com.qmth.examcloud.app.core.exception.ApiException;
import cn.com.qmth.examcloud.app.core.utils.DateUtils;
import cn.com.qmth.examcloud.app.core.utils.HttpUtils;
import cn.com.qmth.examcloud.app.core.utils.JsonMapper;
import cn.com.qmth.examcloud.app.model.Constants;
import cn.com.qmth.examcloud.app.model.Result;
import cn.com.qmth.examcloud.app.service.CoreOeService;
import cn.com.qmth.examcloud.app.core.SysProperty;
import okhttp3.FormBody;
import okhttp3.MediaType;
import okhttp3.MultipartBody;
import okhttp3.RequestBody;
import org.apache.commons.io.FilenameUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static cn.com.qmth.examcloud.app.model.Constants.FILE_TYPE_PDF;
import static cn.com.qmth.examcloud.app.model.Constants.FILE_TYPE_ZIP;

/**
 * 网考业务服务接口
 *
 * @author: fengdesheng
 * @since: 2018/7/16
 */
@Service
public class CoreOeServiceImpl implements CoreOeService {
    private static Logger log = LoggerFactory.getLogger(CoreOeServiceImpl.class);
    @Autowired
    private SysProperty sysProperty;

    @Override
    public Result getCurrentTime(String key, String token) throws Exception {
        //封装请求参数
        final String requestUrl = String.format("%s/api/ecs_oe_student/examControl/currentTime", sysProperty.getCoreOeStudentUrl());
        Result<String> result = HttpUtils.doGet(requestUrl, key, token);
        if (result.isSuccess()) {
            //转换日期格式
            result.setData(DateUtils.formatLongDate(result.getData()));
        }
        return result;
    }

    @Override
    public Result getOfflineExamCourseList(String key, String token) throws Exception {
        //封装请求参数									
        final String requestUrl = String.format("%s/api/ecs_oe_admin/offlineExam/getOfflineCourse", sysProperty.getCoreOeAdminUrl());
        return HttpUtils.doGet(requestUrl, key, token);
    }

    @Override
    public Result startOfflineExamRecord(String key, String token, String examStudentId) throws Exception {
        //封装请求参数									
        final String requestUrl = String.format("%s/api/ecs_oe_admin/offlineExam/startOfflineExam?examStudentId=%s", sysProperty.getCoreOeAdminUrl(), examStudentId);
        return HttpUtils.doGet(requestUrl, key, token);
    }

    @Override
    public Result uploadPaperAnswer(String key, String token, String examRecordId, byte[] fileBytes, String fileName, String md5) throws Exception {
        Assert.notNull(examRecordId, "FileName must not be null.");
        Assert.notNull(fileName, "ExamRecordId must not be null.");
        String fileType = FilenameUtils.getExtension(fileName.toLowerCase());
        if (!FILE_TYPE_ZIP.equals(fileType) && !FILE_TYPE_PDF.equals(fileType)) {
            throw new ApiException("FileType must be zip or pdf.");
        }
        if (fileBytes.length == 0) {
            throw new ApiException("File must be not empty.");
        }
        //封装请求参数									
        final String requestUrl = String.format("%s/api/ecs_oe_admin/offlineExam/submitPaper?examRecordDataId=%s", sysProperty.getCoreOeAdminUrl(), examRecordId);
        MultipartBody.Builder form = new MultipartBody.Builder().setType(MultipartBody.FORM);
        RequestBody body = RequestBody.create(MediaType.parse("application/octet-stream"), fileBytes);
        form.addFormDataPart("file", fileName, body);
        form.addFormDataPart("fileType", fileType);
        form.addFormDataPart("md5", md5 != null ? md5 : "");
        return HttpUtils.doPost(requestUrl, form.build(), key, token);
    }

    @Override
    public Result getPracticeExamCourseList(String key, String token, String examId) throws Exception {
        //封装请求参数
        final String requestUrl = String.format("%s/api/ecs_oe_admin/practice/queryPracticeCourseList?examId=%s", sysProperty.getCoreOeAdminUrl(), examId);
        return HttpUtils.doGet(requestUrl, key, token);
    }

    @Override
    public Result getExamRecordHeartbeat(String key, String token) throws Exception {
        //封装请求参数
        final String requestUrl = String.format("%s/api/ecs_oe_student/examControl/examHeartbeat", sysProperty.getCoreOeStudentUrl());
        return HttpUtils.doGet(requestUrl, key, token);
    }

    @Override
    public Result startPracticeExamRecord(String key, String token, String examStudentId) throws Exception {
        //封装请求参数
        final String requestUrl = String.format("%s/api/ecs_oe_student/examControl/startExam?examStudentId=%s", sysProperty.getCoreOeStudentUrl(), examStudentId);
        return HttpUtils.doGet(requestUrl, key, token);
    }

    @Override
    public Result getAdminExamPaperStructList(String key, String token, String examRecordId,String fromCache) throws Exception {
        //封装请求参数
        String requestUrl=null;
        if("1".equals(fromCache)) {
            requestUrl = String.format("%s/api/ecs_oe_admin/examRecordPaperStruct/getExamRecordPaperStruct?fromCache=1&examRecordDataId=%s", sysProperty.getCoreOeAdminUrl(), examRecordId);
        }else {
            requestUrl = String.format("%s/api/ecs_oe_admin/examRecordPaperStruct/getExamRecordPaperStruct?examRecordDataId=%s", sysProperty.getCoreOeAdminUrl(), examRecordId);
        }
        return HttpUtils.doGet(requestUrl, key, token);
    }
    @Override
    public Result getStudentExamPaperStructList(String key, String token, String examRecordId) throws Exception {
        //封装请求参数
        String requestUrl = String.format("%s/api/ecs_oe_student/examRecordPaperStruct/getExamRecordPaperStruct?fromCache=1&examRecordDataId=%s", sysProperty.getCoreOeStudentUrl(), examRecordId);
        return HttpUtils.doGet(requestUrl, key, token);
    }

    @Override
    public Result getExamRecordPaperQuestionList(String key, String token) throws Exception {
        //封装请求参数
        final String requestUrl = String.format("%s/api/ecs_oe_student/examQuestion/findExamQuestionList", sysProperty.getCoreOeStudentUrl());
        return HttpUtils.doGet(requestUrl, key, token);
    }

    @Override
    public Result getExamRecordPaperQuestionDetail(String key, String token, String questionId, String examRecordId) throws Exception {
        //封装请求参数
        final String requestUrl = String.format("%s/api/ecs_oe_student/examQuestion/getQuestionContent?questionId=%s&exam_record_id=%s", sysProperty.getCoreOeStudentUrl(), questionId, examRecordId);
        return HttpUtils.doGet(requestUrl, key, token);
    }

    @Override
    public Result updateExamRecordPaperQuestionAnswer(String key, String token, Integer order, String studentAnswer) throws Exception {
        //封装请求参数
        final String requestUrl = String.format("%s/api/ecs_oe_student/examQuestion/submitQuestionAnswer", sysProperty.getCoreOeStudentUrl());
        List<Map> paramsList = new ArrayList<Map>();
        Map<String, Object> params = new HashMap<>();
        params.put("order", order);
        params.put("studentAnswer", studentAnswer);
        paramsList.add(params);
        String json = new JsonMapper().toJson(paramsList);
        RequestBody formBody = FormBody.create(MediaType.parse(Constants.CHARSET_JSON_UTF8), json);
        return HttpUtils.doPost(requestUrl, formBody, key, token);
    }

    @Override
    public Result submitPracticeExamRecord(String key, String token) throws Exception {
        //封装请求参数
        final String requestUrl = String.format("%s/api/ecs_oe_student/examControl/endExam", sysProperty.getCoreOeStudentUrl());
        return HttpUtils.doGet(requestUrl, key, token);
    }

    @Override
    public Result checkOnlineExamRecord(String key, String token) throws Exception {
        //封装请求参数
        final String requestUrl = String.format("%s/api/ecs_oe_student/examControl/checkExamInProgress", sysProperty.getCoreOeStudentUrl());
        return HttpUtils.doGet(requestUrl, key, token);
    }

    @Override
    public Result getExamRecordPracticeHistoryList(String key, String token, String examStudentId) throws Exception {
        //封装请求参数
        final String requestUrl = String.format("%s/api/ecs_oe_admin/practice/queryPracticeRecordList?examStudentId=%s", sysProperty.getCoreOeAdminUrl(), examStudentId);
        return HttpUtils.doGet(requestUrl, key, token);
    }

    @Override
    public Result getExamRecordTotalInfo(String key, String token, String examRecordId,String fromCache) throws Exception {
        //封装请求参数
        String requestUrl=null;
        if("1".equals(fromCache)) {
            requestUrl = String.format("%s/api/ecs_oe_admin/practice/getPracticeDetailInfo?fromCache=1&examRecordDataId=%s", sysProperty.getCoreOeAdminUrl(), examRecordId);
        }else {
            requestUrl = String.format("%s/api/ecs_oe_admin/practice/getPracticeDetailInfo?examRecordDataId=%s", sysProperty.getCoreOeAdminUrl(), examRecordId);
        }
        return HttpUtils.doGet(requestUrl, key, token);
    }

    @Override
    public Result getExamRecordQuestionDetailList(String key, String token, String examRecordId) throws Exception {
        //封装请求参数
        final String requestUrl = String.format("%s/api/ecs_oe/report/exam_detail_report/paper_question?exam_record_id=%s", sysProperty.getCoreOeStudentUrl(), examRecordId);
        return HttpUtils.doGet(requestUrl, key, token);
    }

    @Override
    public Result getExamRecordQuestionAudioPlayTimes(String key, String token, String questionId) throws Exception {
        //封装请求参数
        final String requestUrl = String.format("%s/api/exam_question_playtimes?questionId=%s", sysProperty.getCoreOeStudentUrl(), questionId);
        return HttpUtils.doGet(requestUrl, key, token);
    }

    @Override
    public Result updateExamRecordQuestionAudioPlayTimes(String key, String token, String questionId, String mediaName) throws Exception {
        //封装请求参数
        final String requestUrl = String.format("%s/api/exam_question_playtimes", sysProperty.getCoreOeStudentUrl());
        RequestBody formBody = new FormBody.Builder()
                .add("questionId", questionId)
                .add("mediaName", mediaName)
                .build();
        return HttpUtils.doPost(requestUrl, formBody, key, token);
    }

    @Override
    public Result findExamRecordDataEntity(String key, String token, Long examRecordDataId,String fromCache) throws Exception {
        //封装请求参数
        String requestUrl=null;
        if("1".equals(fromCache)) {
            requestUrl = String.format("%s/api/ecs_oe_admin/exam/record/data/findExamRecordDataEntity?fromCache=1&examRecordDataId=%s", sysProperty.getCoreOeAdminUrl(), examRecordDataId);
        }else {
            requestUrl = String.format("%s/api/ecs_oe_admin/exam/record/data/findExamRecordDataEntity?examRecordDataId=%s", sysProperty.getCoreOeAdminUrl(), examRecordDataId);
        }
        Result<String> result = HttpUtils.doGet(requestUrl, key, token);
        if (result.isSuccess()) {
            //过滤掉为空的属性
            result.setData(HttpUtils.filterNullAttributes(result.getData()));
        }
        return result;
    }

    @Override
    public Result getExamRecordQuestions(String key, String token, Long examRecordDataId,String fromCache) throws Exception {
        //封装请求参数
        String requestUrl=null;
        if("1".equals(fromCache)) {
            requestUrl = String.format("%s/api/ecs_oe_admin/examRecordQuestions/getExamRecordQuestions?fromCache=1&examRecordDataId=%s", sysProperty.getCoreOeAdminUrl(), examRecordDataId);
        }else {
            requestUrl = String.format("%s/api/ecs_oe_admin/examRecordQuestions/getExamRecordQuestions?examRecordDataId=%s", sysProperty.getCoreOeAdminUrl(), examRecordDataId);
        }
        Result<String> result = HttpUtils.doGet(requestUrl, key, token);
        if (result.isSuccess()) {
            //过滤掉为空的属性
            result.setData(HttpUtils.filterNullAttributes(result.getData()));
        }
        return result;
    }

    @Override
    public Result getExamPropertyFromCacheByStudentSession(String key, String token,Long examId,String keys) throws Exception {
        //封装请求参数
        final String requestUrl = String.format("%s/api/ecs_exam_work/exam/getExamPropertyFromCacheByStudentSession/"+examId+"/"+keys, sysProperty.getCoreExamWorkUrl());
        Result<String> result = HttpUtils.doGet(requestUrl, key, token);
        return result;
    }

    @Override
    public Result getPaperById(String key, String token, String paperId) throws Exception {
        final String requestUrl = String.format("%s/api/ecs_ques/paper/"+paperId, sysProperty.getCoreQuestionUrl());
        Result<String> result = HttpUtils.doGet(requestUrl, key, token);
        return result;
    }
}