/*
 * *************************************************
 * Copyright (c) 2018 QMTH. All Rights Reserved.
 * Created by Deason on 2018-07-31 17:19:06.
 * *************************************************
 */

package cn.com.qmth.examcloud.app.service.impl;

import cn.com.qmth.examcloud.app.core.utils.HttpUtils;
import cn.com.qmth.examcloud.app.core.utils.JsonMapper;
import cn.com.qmth.examcloud.app.model.Constants;
import cn.com.qmth.examcloud.app.model.ResBody;
import cn.com.qmth.examcloud.app.model.Result;
import cn.com.qmth.examcloud.app.core.SysProperty;
import cn.com.qmth.examcloud.app.service.CoreQuestionService;
import okhttp3.FormBody;
import okhttp3.MediaType;
import okhttp3.RequestBody;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.net.URLEncoder;
import java.util.HashMap;
import java.util.Map;

/**
 * 题库业务服务接口
 *
 * @author: fengdesheng
 * @since: 2018/7/16
 */
@Service
public class CoreQuestionServiceImpl implements CoreQuestionService {
    private static Logger log = LoggerFactory.getLogger(CoreQuestionServiceImpl.class);
    @Autowired
    private SysProperty sysProperty;

    @Override
    public String downloadPaper(String key, String token, String paperId, String orgName) throws Exception {
        //封装请求参数
        orgName = URLEncoder.encode(orgName, "utf-8");
        return String.format("%s/api/ecs_ques/paper/export/%s/PAPER/%s/%s/offLine?$key=%s&$token=%s", sysProperty.getCoreQuestionUrl(), paperId, orgName, paperId, key, token);
    }

    @Override
    public Result getPaperDetail(String key, String token, String paperId) throws Exception {
        //封装请求参数
        final String requestUrl = String.format("%s/api/ecs_ques/paper/%s", sysProperty.getCoreQuestionUrl(), paperId);
        Result<String> result = HttpUtils.doGet(requestUrl, key, token);
        if (result.isSuccess()) {
            //过滤掉为空的属性
            result.setData(HttpUtils.filterNullAttributes(result.getData()));
        }
        return result;
    }

    @Override
    public Result getQuestion(String key, String token, String courseCode, Long examId, String groupCode, String questionId) throws Exception {
        //封装请求参数
        final String requestUrl = String.format("%s/api/ecs_ques/default_question/question", sysProperty.getCoreQuestionUrl());
        Map<String, Object> params = new HashMap<>();
        params.put("courseCode", courseCode);
        params.put("examId", examId);
        params.put("groupCode", groupCode);
        params.put("questionId", questionId);
        String json = new JsonMapper().toJson(params);
        RequestBody formBody = FormBody.create(MediaType.parse(Constants.CHARSET_JSON_UTF8), json);
        //发送短信
        Result<String> result = HttpUtils.doPost(requestUrl, formBody, key, token);
        return this.parseResult(result);
    }

    private Result<String> parseResult(Result<String> result) {
        if (!result.isSuccess()) {
            return result;
        }
        ResBody body = new JsonMapper().fromJson(result.getData(), ResBody.class);
        if (body != null && body.getSuccess() != null) {
            if (body.getSuccess() == true) {
                return new Result().success();
            } else {
                return new Result().error(body.getReturnMsg());
            }
        }
        return result;
    }

}