/*
 * *************************************************
 * Copyright (c) 2018 QMTH. All Rights Reserved.
 * Created by Deason on 2018-07-31 17:14:51.
 * *************************************************
 */

package cn.com.qmth.examcloud.app.service.impl;

import cn.com.qmth.examcloud.app.dao.DeviceRecordRepository;
import cn.com.qmth.examcloud.app.model.DeviceRecord;
import cn.com.qmth.examcloud.app.model.DeviceRecordQuery;
import cn.com.qmth.examcloud.app.model.Result;
import cn.com.qmth.examcloud.app.service.DeviceRecordService;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Sort;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import javax.persistence.criteria.Predicate;
import java.util.ArrayList;
import java.util.List;

/**
 * 设备访问记录Service
 *
 * @author: fengdesheng
 * @since: 2018/7/16
 */
@Service
public class DeviceRecordServiceImpl implements DeviceRecordService {
    private static Logger log = LoggerFactory.getLogger(DeviceRecordServiceImpl.class);
    @Autowired
    private DeviceRecordRepository deviceRecordRepository;

    @SuppressWarnings("unchecked")
    @Override
    public Result<Page<DeviceRecord>> getDeviceRecordList(DeviceRecordQuery params) throws Exception {
        Specification<DeviceRecord> spec = (Specification<DeviceRecord>) (root, query, builder) -> {
            List<Predicate> predicates = new ArrayList<>();
            if (StringUtils.isNotBlank(params.getSystem())) {
                predicates.add(builder.equal(root.get("system"), params.getSystem().trim()));
            }
            if (StringUtils.isNotBlank(params.getDeviceId())) {
                predicates.add(builder.equal(root.get("deviceId"), params.getDeviceId().trim()));
            }
            if (StringUtils.isNotBlank(params.getNetType())) {
                predicates.add(builder.equal(root.get("netType"), params.getNetType().trim()));
            }
            if (StringUtils.isNotBlank(params.getBrand())) {
                predicates.add(builder.equal(root.get("brand"), params.getBrand().trim()));
            }
            if (StringUtils.isNotBlank(params.getModel())) {
                predicates.add(builder.equal(root.get("model"), params.getModel().trim()));
            }
            if (StringUtils.isNotBlank(params.getSysVersion())) {
                predicates.add(builder.equal(root.get("sysVersion"), params.getSysVersion().trim()));
            }
            if (StringUtils.isNotBlank(params.getAppVersion())) {
                predicates.add(builder.equal(root.get("appVersion"), params.getAppVersion().trim()));
            }
            if (StringUtils.isNotBlank(params.getAppCode())) {
                predicates.add(builder.equal(root.get("appCode"), params.getAppCode().trim()));
            }
            if (StringUtils.isNotBlank(params.getPatchCode())) {
                predicates.add(builder.equal(root.get("patchCode"), params.getPatchCode().trim()));
            }
            if (StringUtils.isNotBlank(params.getAccount())) {
                predicates.add(builder.equal(root.get("account"), params.getAccount().trim()));
            }
            if (StringUtils.isNotBlank(params.getLoginKey())) {
                predicates.add(builder.equal(root.get("loginKey"), params.getLoginKey().trim()));
            }
            if (StringUtils.isNotBlank(params.getLoginToken())) {
                predicates.add(builder.equal(root.get("loginToken"), params.getLoginToken().trim()));
            }
            if (StringUtils.isNotBlank(params.getUrl())) {
                predicates.add(builder.like(root.get("url"), "%" + params.getUrl().trim() + "%"));
            }
            if (StringUtils.isNotBlank(params.getIp())) {
                predicates.add(builder.equal(root.get("ip"), params.getIp().trim()));
            }
            return builder.and(predicates.toArray(new Predicate[predicates.size()]));
        };

        Sort sort = new Sort(Sort.Direction.DESC, "id");
        Pageable pageable = PageRequest.of(params.getPageNo() - 1, params.getPageSize(), sort);
        Page<DeviceRecord> page = deviceRecordRepository.findAll(spec, pageable);
        return new Result<>().success(page);
    }

    @Async
    @Override
    public void addDeviceRecord(DeviceRecord deviceRecord) throws Exception {
        Assert.notNull(deviceRecord, "DeviceRecord must not be null.");
        deviceRecordRepository.save(deviceRecord);
    }

}